// GMap.cpp: implementation of the GMap class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "GObject.h"
#include "GMap.h"
#include "GView.h"
#include "GRun.h"


/*////////////////////////////////////////////////////////////////////////////////
  Konstruktor
*/
GMap::GMap()
{
	m_bGround = NULL;
	m_bWalk   = NULL;

	int i;
	for(i = 0; i < MAX_OBJECTS_ON_MAP; i++)
		m_objects[i] = NULL;
	m_cobjects = 0;

	for(i = 0; i < MAX_STREL_ON_MAP; i++)
		m_strely[i].m_active = false;
	m_strel = 0;

	for(i = 0; i < MAX_POWERUPS_ON_MAP; i++)
			m_power[i] = NULL;

	m_nextpwr = 0;


	m_sx = 1280;
	m_sy = 1280;
//	m_mode = TYPE_SMALL;
	m_mode = TYPE_LARGE;

}

/*////////////////////////////////////////////////////////////////////////////////
  Destruktor
*/
GMap::~GMap()
{
	DestroyArrays();
}

/*////////////////////////////////////////////////////////////////////////////////
  Smaze vsechna data a pripravi se na nove zadavani
  road   :  pocet bodu silnice
  object :  pocet objektu
*/
void GMap::Reset(int X, int Y)
{
	DestroyArrays();
	m_sx = X;
	m_sy = Y;
}

/*////////////////////////////////////////////////////////////////////////////////
  Oznamuje, ze byla hotova inicializace (pri nacitani ze souboru)
*/
void GMap::InitDone()
{
	if (m_mode == TYPE_LARGE)
		FillLargeGround();
}

/*////////////////////////////////////////////////////////////////////////////////
  Prida dalsi bod silnice
*/


/*////////////////////////////////////////////////////////////////////////////////
  Prida dalsi objekt
*/
void GMap::AddObject(st_GFile_Object ob)
{
	if (m_cobjects < MAX_OBJECTS_ON_MAP) {
		m_objects[m_cobjects] = new GObject;
		m_objects[m_cobjects]->Init(ob.x, ob.y, ob.ID);
		m_cobjects++;
	}
}

/*////////////////////////////////////////////////////////////////////////////////
  Znici alokovane bitmapy
*/


/*////////////////////////////////////////////////////////////////////////////////
  Kresli mapu - podklad (travu a silnice)
  CX  :  x-ova souradnice kamery
  CY  :  y-ova souradnice kamery
*/
void GMap::Draw(int CX, int CY, bool stin)
{
	//TODO: dodelat mapu
	int i;
	// podklad
	if (m_mode == TYPE_SMALL)
		blit(m_bGround, hscreen.m_back, (CX - DF_X/2)%40, (CY - DF_Y/2)%40, 0, 0, DF_X, DF_Y);
	else 
		blit(m_bGround, hscreen.m_back, (CX - DF_X/2), (CY - DF_Y/2), 0, 0, DF_X, DF_Y);

	for(i = 0; i < MAX_POWERUPS_ON_MAP; i++)
		if (m_power[i] != NULL) m_power[i]->Draw(CX, CY);

	// kresleni strel
	for(i = 0; i < MAX_STREL_ON_MAP; i++)
		if(m_strely[i].m_active) m_strely[i].Draw(CX, CY, stin);
}

/*////////////////////////////////////////////////////////////////////////////////
  Kresli mapu - objekty
  CX  :  x-ova souradnice kamery
  CY  :  y-ova souradnice kamery
  hi  :  vyskova hladina 
*/
//DEL void GMap::DrawObj(int CX, int CY, int hi, bool stin)
//DEL {
//DEL 	for( int i = 0; i < m_cobjects; i++)
//DEL 		if (m_objects[i] != NULL) m_objects[i]->Draw(hscreen.m_back, CX, CY, hi, stin);
//DEL }

/*////////////////////////////////////////////////////////////////////////////////
  Nacte graficka data
*/
bool GMap::LoadGFX()
{
	Destroy();

	PALETTE pal;


	m_bGround = load_bitmap("lev1.bmp",pal);
	m_bWalk   = load_bitmap("lev1b.pcx",pal);
	return true;
}
/*////////////////////////////////////////////////////////////////////////////////
  Pohne vsemi objekty
*/
void GMap::Move()
{
	int i;
	for(i = 0; i < m_cobjects; i++)
		if (m_objects[i] != NULL) m_objects[i]->Move();

	if(--m_nextpwr <= 0) 
		NewPowerUp();  // jak dlouho zbyva do pridani dalsiho powerupu


	m_strelA = 0;
	for(i = 0; i < MAX_STREL_ON_MAP; i++)
		if (m_strely[i].m_active) {
			m_strely[i].Move(this);
			m_strelA++;
		}
}

/*////////////////////////////////////////////////////////////////////////////////
  Rusi pole m_objects
*/
void GMap::DestroyArrays()
{
	int i;

	// ruseni objektu
	for(i = 0; i < MAX_OBJECTS_ON_MAP; i++)
		if (m_objects[i] != NULL) {
			delete m_objects[i];
			m_objects[i] = NULL;
		}
		m_cobjects = 0;

	for(i = 0; i < MAX_STREL_ON_MAP; i++)
		m_strely[i].m_active = false;
	m_strel = 0;

	for(i = 0; i < MAX_POWERUPS_ON_MAP; i++)
		if (m_power[i] != NULL) {
			delete m_power[i];
			m_power[i] = NULL;
		}
}

/*////////////////////////////////////////////////////////////////////////////////
  Testuje narazy - bere vsechna auticka a zkousi je na vsechny objekty
*/

void GMap::FillLargeGround(){

}

/*////////////////////////////////////////////////////////////////////////////////
  Nici bitmapy 
*/
void GMap::Destroy()
{
	if (m_bGround != NULL)
		destroy_bitmap(m_bGround);
	m_bGround = NULL;
	if (m_bWalk != NULL)
		destroy_bitmap(m_bWalk);
	m_bWalk = NULL;
}

/*////////////////////////////////////////////////////////////////////////////////
  prida novou strelu
*/
void GMap::NewStrela(int x, int y, int angle, int ID, int who, double speed, int zbranID)
{
	m_strel++;
	int i;
	for(i = 0; i < MAX_STREL_ON_MAP; i++)
		if(!m_strely[i].m_active) break;
	if (i == MAX_STREL_ON_MAP) return;
	m_strely[i].Init(x,y,angle,ID,who, speed, zbranID);
}

/*////////////////////////////////////////////////////////////////////////////////
	Zkusi zasahy do hracu
*/
void GMap::ShootHits(GRun *run)
{
	int i,j = 0, who = 0;
	for(i = 0; i < MAX_STREL_ON_MAP; i++)
		if (m_strely[i].m_active) {
			who = m_strely[i].m_who;
			for( int k = 0; k < run->m_soldiers; k++) {
				if (!m_strely[i].m_active) break;
				if (Hit(&run->m_sold[k], &m_strely[i], run)) {
					if (!run->m_multi) {
						InsertKilled(run->m_sold[k].m_x, run->m_sold[k].m_y);
						if((++run->m_sold[who].m_frags)%10 == 0) 
							run->m_view.AddFragsMess(who, run);
					}
					else 
					if (k == 0) {
						run->m_net.SendFrag(who);
						InsertKilled(run->m_sold[k].m_x, run->m_sold[k].m_y);
					}
				}
			}
			j++;
			if(j == m_strelA) break; // uz byly zpracovany vsechny strely
		}
}

/*////////////////////////////////////////////////////////////////////////////////
	Zda se dana strela trefila
*/
bool GMap::Hit(GSoldier *sold, GStrela *strela, GRun *run)
{
	GStrela *str = strela;
	bool ret = false;

	// aby hrac nezabil sam sebe
	if (sold->m_health <= 0) return false;
	if (str->m_who == sold->m_index) return false;

   int dist = pow((pow(abs(sold->m_x-str->m_x),2)+pow(abs(sold->m_y-str->m_y),2)),0.5);

	if (dist > str->m_dist + sold->m_dist) return false; // strela minula

	// zasah
	if ((ret = sold->HitBy(str->GetDemage(this),str->m_who)) == true) {
		// vojak byl zabit - tak to oznamime
		run->m_view.AddKillMessage(str->m_who, sold->m_index, run);
	}

	// umazani strely 
	strela->m_active = false;

	return ret;
}

/*////////////////////////////////////////////////////////////////////////////////
	Prida do hry jeden dalsi bonus
*/
void GMap::NewPowerUp()
{
	int i;
	for(i = 0; i < MAX_POWERUPS_ON_MAP; i++) 
		if (m_power[i] == NULL) break;
	if (i == MAX_POWERUPS_ON_MAP) return;

	m_power[i] = new GPowerUp();

	int x,y;
	GetFreePos(&x, &y, 20);

	m_power[i]->Init(x, y, rand()%MAX_POWERUPS);

	m_nextpwr = 5+rand()%10;
}

/*////////////////////////////////////////////////////////////////////////////////
	Zajistuje zbirani powerupu
*/
void GMap::PickPowerUps(GRun *run)
{
	int i,j;
	for(i = 0; i < MAX_POWERUPS_ON_MAP; i++)
		if (m_power[i] != NULL)
			for( int k = 0; k < run->m_soldiers; k++) {
				if (m_power[i] == NULL) break;
				j = m_power[i]->m_ID;
				if (m_power[i]->Picked(&run->m_sold[k], &m_power[i])) {
					if (k == run->m_monsol) run->m_view.AddBonMessage(j);
					break;
				}
			}
}

/*////////////////////////////////////////////////////////////////////////////////
	Vlozi mrtvolu vojaka na zem
*/
void GMap::InsertKilled(int x, int y)
{
	rotate_sprite(m_bGround, b_deads[rand()%MAX_DEADS_BITMAPS],  x-20, y-20, itofix(rand()%256));
}

/*////////////////////////////////////////////////////////////////////////////////
	Vrati volnou pozici na mape
*/
void GMap::GetFreePos(int *x, int *y, int dst)
{
	do {
		*x = rand()%1280;
		*y = rand()%1200;
	} while (!CanWalk(*x, *y, dst));
}

/*////////////////////////////////////////////////////////////////////////////////
	Vrati zda je dane policko pruchodne
*/
bool GMap::CanWalk(int x, int y, int dst)
{
	if (CanWalk(x,y) && CanWalk(x+dst,y+dst) &&
		 CanWalk(x-dst,y+dst) && CanWalk(x+dst,y-dst) &&
		 CanWalk(x-dst,y-dst)) return true;
	return false;
}

/*////////////////////////////////////////////////////////////////////////////////
	Vrati zda je dana pozice pristrelna
*/

bool GMap::CanShoot(int x, int y, int dst)
{
	if (CanShoot(x,y) && CanShoot(x+dst,y+dst) &&
		 CanShoot(x-dst,y+dst) && CanShoot(x+dst,y-dst) &&
		 CanShoot(x-dst,y-dst)) return true;
	return false;

}

bool GMap::CanWalk(int x, int y)
{
	int col = getpixel(m_bWalk, x, y);
	if (col != makecol(255,255,255)) return false;
	return true;
}

bool GMap::CanShoot(int x, int y)
{
	int col = getpixel(m_bWalk, x, y);
	if (col == -1 || col == 0) return false;
	return true;
}
