// GRun.cpp: implementation of the GRun class.
//
//////////////////////////////////////////////////////////////////////

#include "stdh.h"

#include "GSoldier.h"
#include "GView.h"
#include "GObject.h"
#include "GFile.h"
#include "GMap.h"
#include "GRun.h"
#include "zbrane.h"

#include "MultiPlay.h"

#define MSG_SOLD_STAT    1
#define MSG_CONTROL_STAT 2

const char ai_names[AI_NAMES][30] = {
	"HIPPO",  "DOOM",  "KULATOR", "QUAKOSH", "STUPIDIO",  "ZEUS", "TERMINATOR",
	"GATES",  "LINUX", "ERROR",    "HANZ",    "HITLER",  "STALIN", "GORBACOV",
	"ZEMAN",  "VUL",  "VINETOU",   "BILL",     "SAM",    "MOLOCH",  "RAMBO", 
	"PRASAK", "LENIN", "WINZIP", "MIRCOSOFT", "VETRELEC", "KULIC",  "RAYEN" 
};

/*////////////////////////////////////////////////////////////////////////////////
	NULL-ovaci konstruktor
*/
GRun::GRun()
{
	int i;


	m_game_speed = 5; // rychlost 20 snimku za sekundu

	for (i = 0; i < MAX_SOLD_BITMAPS; i++) {
		b_solds[i] = NULL;
		b_shadows[i] = NULL;
	}

	for (i = 0; i < MAX_DEADS_BITMAPS; i++) 
		b_deads[i] = NULL;

	for (i = 0; i < MAX_OBJECTS_BITMAPS; i++) 
		b_objects[i] = NULL;

	m_stin  = true;

	m_doruststrel = true; // strely dorustaji
	m_maxstrel    = 50;   // na zacatku dostane vojak 50 % vybaveni
}

/*////////////////////////////////////////////////////////////////////////////////
	Rusi ruzne dinamicke obekty
*/
GRun::~GRun()
{
}

/*////////////////////////////////////////////////////////////////////////////////
	Hlavni fce, diky ketre bezi samotna hra ( vsechno to kloubi dohromady )
*/
void GRun::Run()
{
   clear_keybuf();
   hscreen.m_mouse = false;

	net->ScanForPlayers();

	if(m_net.m_connected) {
//		m_soldiers = net->NumPlayers;
		m_soldiers = 1;
		m_multi = true;
	}
	else m_multi = false;
	
	m_monsol = 0;

  tmr = 5; // 50 ms; -> 20 refreshu za sekundu
  do {

		// send data
		if (m_multi) {
			m_net.UpdateSoldiers(this);
			m_net.Send(this);
		}
		
		// zobrazeni
		Draw();

		// nastaveni pocitace
		CompMove();

		// takhle hraje nas hrac
		m_sold[0].SetKeys(key[KEY_UP], key[KEY_DOWN], key[KEY_LEFT], key[KEY_RIGHT], key[KEY_ALTGR], key[KEY_RSHIFT], key[KEY_RCONTROL], key[KEY_L], key[KEY_ENTER]);


		// a jedem
		Move();

	 	if (m_multi)
			m_net.Receive(this);

		while (tmr > 0);
		tmr = 5; // 50 ms; -> 20 refreshu za sekundu

		if (m_multi)
			m_net.Receive(this);

		Crash();

   } while (!key[KEY_ESC]);
}

/*////////////////////////////////////////////////////////////////////////////////
	Rusi vsechny bitmapy
*/
void GRun::Destroy()
{
	int i,j;

	// Ruseni vojaku
	for (i = 0; i < MAX_SOLD_BITMAPS; i++) {
		if (b_solds[i] != NULL) destroy_bitmap(b_solds[i]);
		b_solds[i] = NULL;
		if (b_shadows[i] != NULL) destroy_bitmap(b_shadows[i]);
		b_shadows[i] = NULL;
	}
	//  Ruseni mrtvol
	for (i = 0; i < MAX_DEADS_BITMAPS; i++) {
		if (b_deads[i] != NULL) destroy_bitmap(b_deads[i]);
		b_deads[i] = NULL;
	}
	// Ruseni strel
	for (i = 0; i < MAX_STREL; i++) {
		if (b_strel[i] != NULL) destroy_bitmap(b_strel[i]);
		b_strel[i] = NULL;
	}
	// ruseni zbrani
	for( i = 0; i < MAX_ZBRANE; i++){
		// objekt
		if (b_zbraneB[i] != NULL) destroy_bitmap(b_zbraneB[i]);
		b_zbraneB[i] = NULL;
		for ( j = 0; j < MAX_ZBRAN_ANIM; j++) {
			if (b_zbrane[i][j] != NULL) destroy_bitmap(b_zbrane[i][j]);
			b_zbrane[i][j] = NULL;
		}
	}
	// Ruseni objektu
	for (i = 0; i < MAX_OBJECTS_BITMAPS; i++) {
		if (b_objects[i] != NULL) destroy_bitmap(b_objects[i]);
		b_objects[i] = NULL;
	}
	// Stiny u objekty
	for (i = 0; i < MAX_OBJECTS; i++) {
		if (b_sobjects[i] != NULL) destroy_bitmap(b_sobjects[i]);
		b_sobjects[i] = NULL;
	}
	// ruseni powerupu
	for (i = 0; i < MAX_POWERUPS; i++) {
		if (b_powerups[i] != NULL) destroy_bitmap(b_powerups[i]);
		b_powerups[i] = NULL;
	}

	// ruseni animaci
	for( i = 0; i < MAX_KREV_ANIM; i++){
		if (b_krev[i] != NULL) destroy_bitmap(b_krev[i]);
		b_krev[i] = NULL;

	}
	for( i = 0; i < MAX_STIT_ANIM; i++){
		if (b_stit[i] != NULL) destroy_bitmap(b_stit[i]);
		b_stit[i] = NULL;

	}

	m_view.Destroy();
	m_map.Destroy();

}

/*////////////////////////////////////////////////////////////////////////////////
	zaridi nacteni bitmap ( vraci false pri neuspechu )
*/
bool GRun::LoadGFX()
{
	PALETTE pal;
	
	Destroy(); // radsi to nejrve zlikvidujem ( pro jistotu )
	int i,j;
	char s[60];

 //////////////////////
//	  NACTENI Vojaku   //
 //////////////////////

	for( i = 0; i < MAX_SOLD_BITMAPS; i++){
		// objekt
		sprintf(s,"sold%d.bmp",i+1);
		b_solds[i] = load_bitmap(s,pal);
		if (!b_solds[i]) {
			set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
			allegro_message("Chybi %s",s);
			return false;
		}
	}

 //////////////////////
//	  nacteni mrtvol   //
 //////////////////////
	for( i = 0; i < MAX_DEADS_BITMAPS; i++){
		// objekt
		sprintf(s,"dead%d.bmp",i+1);
		b_deads[i] = load_bitmap(s,pal);
		if (!b_deads[i]) {
			set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
			allegro_message("Chybi %s",s);
			return false;
		}
	}
	
 //////////////////////
//	  NACTENI Strel   //
 //////////////////////
	for( i = 0; i < MAX_STREL; i++){
		// objekt
		sprintf(s,"GStrela%d.bmp",i+1);
		b_strel[i] = load_bitmap(s,pal);
		if (!b_strel[i]) {
			set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
			allegro_message("Chybi %s",s);
			return false;
		}
	}
	
/////////////////////////
//	  NACTENI OBJEKTU   //
/////////////////////////
/*	for( i = 0; i < MAX_OBJECTS; i++){
		// objekt
		sprintf(s,"%s.bmp",Oparams[i].file);
		b_objects[Oparams[i].bindex] = load_bitmap(s,pal);
		if (!b_objects[Oparams[i].bindex]) {
			set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
			allegro_message("Chybi %s",s);
			return false;
		}

		sprintf(s,"%ss.bmp",Oparams[i].file);
		b_sobjects[i] = load_bitmap(s,pal);
		if (!b_sobjects[i]) {
			set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
			allegro_message("Chybi %s",s);
			return false;
		}

		// rozjety
		sprintf(s,"%sB.bmp",Oparams[i].file);
		b_objects[Oparams[i].bindex+1] = load_bitmap(s,pal);
		if (!b_objects[Oparams[i].bindex]) {
			set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
			allegro_message("Chybi %s",s);
			return false;
		}
	}*/
/////////////////////
// nacteni animaci //
/////////////////////
	for( i = 0; i < MAX_KREV_ANIM; i++){
		// objekt
		sprintf(s,"AKrev%d.bmp",i+1);
		b_krev[i] = load_bitmap(s,pal);
		if (!b_krev[i]) {
			set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
			allegro_message("Chybi %s",s);
			return false;
		}
	}
	for( i = 0; i < MAX_STIT_ANIM; i++){
		// objekt
		sprintf(s,"AStit%d.bmp",i+1);
		b_stit[i] = load_bitmap(s,pal);
		if (!b_stit[i]) {
			set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
			allegro_message("Chybi %s",s);
			return false;
		}
	}

 //////////////////////
//	  NACTENI Powerupu //
 //////////////////////
	for( i = 0; i < MAX_POWERUPS; i++){
		// objekt
		sprintf(s,"pup%d.bmp",i+1);
		b_powerups[i] = load_bitmap(s,pal);
		if (!b_powerups[i]) {
			set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
			allegro_message("Chybi %s",s);
			return false;
		}
	}

 //////////////////////
//	  NACTENI Zbrani   //
 //////////////////////
	for( i = 0; i < MAX_ZBRANE; i++){
		// objekt
		sprintf(s,"GZbranB%d.bmp",i+1);
		b_zbraneB[i] = load_bitmap(s,pal);
		if (!b_zbraneB[i]) {
			set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
			allegro_message("Chybi %s",s);
			return false;
		}
		for( j = 0; j < Zparams[i].anim; j++) {
			sprintf(s,"GZbran%d%d.bmp",i+1,j);
			b_zbrane[i][j] = load_bitmap(s,pal);
			if (!b_zbrane[i][j]) {
				set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
				allegro_message("Chybi %s",s);
				return false;
			}
		}
	}

/////////////
// Ostatni //
/////////////
	if (!m_map.LoadGFX()) return false;
	if (!m_view.LoadGFX()) return false;

	fonts = load_datafile("fonts.dat");
	if (fonts == NULL) {
		allegro_message("chybi fonts.dat");
		return false;
	}


	return true;
}

/*////////////////////////////////////////////////////////////////////////////////
	Nacte novou trat
*/
bool GRun::NewRace(char *file)
{
	GFile gfile;
	if(!gfile.OpenFile(file)) {
		gfile.CloseFile();
		return false;
	}
	if(!gfile.LoadData(&m_map)) {
		gfile.CloseFile();
		return false;
	}

	gfile.CloseFile();
	
	return true;
}

/*////////////////////////////////////////////////////////////////////////////////
	Zajisti vykresleni (+ flip screen)
*/
void GRun::Draw()
{
	m_view.UpdateCamera(this);
	m_view.Draw(this);
}

/*////////////////////////////////////////////////////////////////////////////////
	Zajisti pohnuti vsemi objkety
*/
void GRun::Move()
{
	m_map.Move();
	// pocitac
	if (!m_multi)
		for (int i = 1; i < m_soldiers; i++)
			m_sold[i].Move(this);
	// nas hrac
	m_sold[0].Move(this);

	m_map.ShootHits(this);
	m_map.PickPowerUps(this);
}

/*////////////////////////////////////////////////////////////////////////////////
	Testuje narazy
*/
void GRun::Crash()
{
	int k = 0, // promena cyklu
		 l = 0; // pocet narazeni
	
}

/*////////////////////////////////////////////////////////////////////////////////
	Pripravi hru na hru hrac vs. pocitace
*/
void GRun::InitSingle(int players)
{
	m_soldiers = players;
}

/*////////////////////////////////////////////////////////////////////////////////
	Zinicializuje hru
*/
bool GRun::Init()
{
	int i;


	// Vojak
	for(i = 0; i < MAX_PLAYERS; i++) {
		m_sold[i].m_ID = rand()%MAX_SOLDIERS;
		m_sold[i].Init(i, m_map.m_sx, m_map.m_sx, rand()%MAX_ZBRANE, true, 50);
		m_sold[i].m_active = true;
		if (i > 0) strcpy(m_sold[i].m_name, ai_names[rand()%AI_NAMES]);
	}

	m_map.LoadGFX();
	m_map.InitDone();


	m_view.Init(&m_sold[0]);

	m_map.NewPowerUp();
	m_map.NewPowerUp();
	m_map.NewPowerUp();


  if (!m_net.EnterGame(this)) {
		allegro_message("Failed to HOST or JOIN game "); // SMULA !!!!
		return false;
  }

  return true;
}

void GRun::CompMove()
{
	int i;
	if (m_multi) return;
		for(i = 1; i < m_soldiers; i++)
			m_sold[i].CompMove(this, i);
}


