// GRun.cpp: implementation of the GRun class.
//
//////////////////////////////////////////////////////////////////////

#include "stdh.h"

#include "GSoldier.h"
#include "GView.h"
#include "GFile.h"
#include "GMap.h"
#include "GRun.h"
#include "zbrane.h"

#include "MultiPlay.h"

#include "inifile.h"
extern st_inifile ini;


#define MSG_SOLD_STAT    1
#define MSG_CONTROL_STAT 2

#define AI_NAMES  70


const char ai_names[AI_NAMES][40] = {
	"HIPPO",     "DOOM",     "KULATOR",   "QUAKOSH",       "STUPIDIO",     "ZEUS",       "TERMINATOR",
	"GATES",     "LINUX",    "ERROR",     "HANZ",          "HITLER",       "STALIN",     "GORBACOV",
	"ZEMAN",     "VUL",      "VINETOU",   "BILL",          "SAM",          "MOLOCH",     "RAMBO", 
	"PRASAK",    "LENIN",    "WINZIP",    "MIRCOSOFT",     "VETRELEC",     "KULIC",      "RAYEN",  
   "ROBOCOP",   "KUDLANKA", "KRAKATICE", "ZRALOK",        "PREDATOR",     "JAMES BOND", "DART VADER",
	"SKYWALKER", "NAPOLEON", "CEZAR",     "CISAR",         "PAPEZ",        "LEWINSKA",   "GOLEM", 
	"RADEGAST",  "SVATOVIT", "PERUN",     "GAMBRINUS",     "IMPERATOR",    "BUDWAR",     "STAROPRAMEN",
	"NEPTUN",    "ROLAND",   "MR. BEAN",  "KING KONG",     "KAREL IV.",    "KOMUNISTA",  "DEWIL",
	"CARTMAN",   "KENNY",    "KYLE",      "HOMER SIMPSON", "BART SIMPSON", "ESMERLANDA", "LUNETIC",
	"HANIBAL",   "KYKLOP",   "KLEOPATRA", "ODYSSEUS",      "NAUTILUS",     "LIKVIDATOR", "ALEXANDR VELIKY",
};

/*  DALSI JMENA v poradi
	"JACKSON", "KOLIMBUS", "NEDVED", "TARZAN", "SERCHAN", 
	"DIVOKY ANDEL", "RAMPA MC KVAK", "KRAKONOS", "KASSANDRA", "MILA GROSS", 
	"KLAUS", "SLADEK", "GAGARIN", "SADAM HUSAJN", "ODPADLIK", "STAN", "WENDY", "CHEF",
	"KARKULKA", "SCHIMANSKY", "COLA", "KOFOLA", "MRAKOPLAS", "SIROKO", "HERA", "RAMA",
	"FLORA", "PEPINO", "LEO", 
	
	  "OTTO VON RICHTERVHOVEN",
*/

/*////////////////////////////////////////////////////////////////////////////////
	NULL-ovaci konstruktor
*/
GRun::GRun()
{
	int i;

	m_autorun = false;

	m_game_speed = 5; // rychlost 20 snimku za sekundu

	for (i = 0; i < MAX_SOLD_BITMAPS; i++) {
		b_solds[i] = NULL;
	}

	for (i = 0; i < MAX_DEADS_BITMAPS; i++) 
		b_deads[i] = NULL;


	m_stin  = true;

	m_doruststrel = true; // strely dorustaji
	m_maxstrel    = 50;   // na zacatku dostane vojak 50 % vybaveni
}

/*////////////////////////////////////////////////////////////////////////////////
	Rusi ruzne dinamicke obekty
*/
GRun::~GRun()
{
}

/*////////////////////////////////////////////////////////////////////////////////
	Hlavni fce, diky ketre bezi samotna hra ( vsechno to kloubi dohromady )
*/
void GRun::Run()
{
   clear_keybuf();
   hscreen.m_mouse = false;

	net->ScanForPlayers();

	if(m_net.m_connected) {
		m_soldiers = 1;
		m_multi = true;
	}
	else m_multi = false;
	
	m_multi = false;

	m_monsol = 0;

	cdplay.Play();
	tmr  = 5; // 50 ms; -> 20 refreshu za sekundu
	tmr2 = 400;  // timer pro CD
	do {
		if (tmr2 == 0) {
			cdplay.Play();
			tmr2 = 400;  // timer pro CD
		}

		// send data
		if (m_multi) {
			m_net.UpdateSoldiers(this);
			m_net.Send(this);
		}
		
		// zobrazeni
		Draw();

		// nastaveni pocitace
		CompMove();

		ProcessKeys();

		// takhle hraje nas hrac
		SetCurPlCtrl();


		// a jedem
		Move();

	 	if (m_multi)
			m_net.Receive(this);

		while (tmr > 0);
		tmr = 5; // 50 ms; -> 20 refreshu za sekundu

		if (m_multi)
			m_net.Receive(this);

		Crash();

   } while (!key[KEY_ESC]);
	cdplay.Pause();
}

/*////////////////////////////////////////////////////////////////////////////////
	Rusi vsechny bitmapy
*/
void GRun::Destroy()
{
	int i,j;

	// Ruseni vojaku
	DestroyBitmaps(&b_solds[0], MAX_SOLD_BITMAPS);

	// Ruseni bonusu
	DestroyBitmaps(&b_s_bonus[0], MAX_BONUS);

	//  Ruseni mrtvol
	DestroyBitmaps(&b_deads[0], MAX_DEADS_BITMAPS);

	// Ruseni nohou
	for( i = 0; i < MAX_RUN_BITMAPS; i++){
		if (b_s_run[i] != NULL) destroy_bitmap(b_s_run[i]);
		b_s_run[i] = NULL;
	}
	for( i = 0; i < MAX_WALK_BITMAPS; i++){
		if (b_s_walk[i] != NULL) destroy_bitmap(b_s_walk[i]);
		b_s_walk[i] = NULL;
	}
	// Ruseni wody
	for( i = 0; i < MAX_WATER_BITMAPS; i++){
		if (b_s_water[i] != NULL) destroy_bitmap(b_s_water[i]);
		b_s_water[i] = NULL;
	}
	// Ruseni strel
	for (i = 0; i < MAX_STREL; i++) {
		if (b_strel[i] != NULL) destroy_bitmap(b_strel[i]);
		b_strel[i] = NULL;
	}
	for( i = 0; i < MAX_KOUR; i++){
		if (b_kour[i] != NULL) destroy_bitmap(b_kour[i]);
		b_kour[i] = NULL;
	}
	// ruseni zbrani
	for( i = 0; i < MAX_ZBRANE; i++){
		// objekt
		if (b_zbraneB[i] != NULL) destroy_bitmap(b_zbraneB[i]);
		b_zbraneB[i] = NULL;
		for ( j = 0; j < MAX_ZBRAN_ANIM; j++) {
			if (b_zbrane[i][j] != NULL) destroy_bitmap(b_zbrane[i][j]);
			b_zbrane[i][j] = NULL;
		}
	}
	// ruseni powerupu
	for (i = 0; i < MAX_POWERUPS; i++) {
		if (b_powerups[i] != NULL) destroy_bitmap(b_powerups[i]);
		b_powerups[i] = NULL;
	}

	// ruseni animaci
	for( i = 0; i < MAX_KREV_ANIM; i++){
		if (b_krev[i] != NULL) destroy_bitmap(b_krev[i]);
		b_krev[i] = NULL;

	}
	for( i = 0; i < MAX_STIT_ANIM; i++){
		if (b_stit[i] != NULL) destroy_bitmap(b_stit[i]);
		b_stit[i] = NULL;

	}

	m_view.Destroy();
	m_map.Destroy();

}

/*////////////////////////////////////////////////////////////////////////////////
	zaridi nacteni bitmap ( vraci false pri neuspechu )
*/
bool GRun::LoadGFX()
{
	
	Destroy(); // radsi to nejrve zlikvidujem ( pro jistotu )
	int i;
	char s[60];

	// vojaci
	if (!LoadBitmaps(&b_solds[0], MAX_SOLD_BITMAPS, 1, "sold%d.bmp")) return false;

	// mrtvoly
	if (!LoadBitmaps(&b_deads[0], MAX_DEADS_BITMAPS, 1, "dead%d.bmp")) return false;

	// nacteni nohou  pri behu
	if (!LoadBitmaps(&b_s_run[0], MAX_RUN_BITMAPS, 0, "SR%.2d.bmp")) return false;

	// normalni nohy
	if (!LoadBitmaps(&b_s_walk[0], MAX_WALK_BITMAPS, 0, "S%d.bmp")) return false;

	// animace wody
	if (!LoadBitmaps(&b_s_water[0], MAX_WATER_BITMAPS, 1, "AWoda%d.bmp")) return false;

	// strely
	if (!LoadBitmaps(&b_strel[0], MAX_STREL, 1, "GStrela%d.bmp")) return false;

	// kour
	if (!LoadBitmaps(&b_kour[0], MAX_KOUR, 1, "GKour%d.bmp")) return false;
	
	// strikajici krev
	if (!LoadBitmaps(&b_krev[0], MAX_KREV_ANIM, 1, "AKrev%d.bmp")) return false;

	// blikajici stit
	if (!LoadBitmaps(&b_stit[0], MAX_STIT_ANIM, 1, "AStit%d.bmp")) return false;

	// power-upy
	if (!LoadBitmaps(&b_powerups[0], MAX_POWERUPS, 1, "pup%d.bmp")) return false;

	// velke zbrane
	if (!LoadBitmaps(&b_zbraneB[0], MAX_ZBRANE, 1, "GZbranB%d.bmp")) return false;

	// bonusy u vojaku
	if (!LoadBitmaps(&b_s_bonus[0], MAX_BONUS, 1, "bon_%d.bmp")) return false;
	

	// animace malych zbrani
	for( i = 0; i < MAX_ZBRANE; i++){
		sprintf(s,"GZbran%d%%d.bmp",i+1);
		if (!LoadBitmaps(&b_zbrane[i][0], Zparams[i].anim, 0, s)) return false;
	}

/////////////
// Ostatni //
/////////////
	if (!m_map.LoadGFX()) return false;
	if (!m_view.LoadGFX(this)) return false;

	fonts = load_datafile("fonts.dat");
	if (fonts == NULL) {
		allegro_message("chybi fonts.dat");
		return false;
	}

	return true;
}

/*////////////////////////////////////////////////////////////////////////////////
	Nacte novou trat
*/
bool GRun::NewRace(char *file)
{
	GFile gfile;
	if(!gfile.OpenFile(file)) {
		gfile.CloseFile();
		return false;
	}
	if(!gfile.LoadData(&m_map)) {
		gfile.CloseFile();
		return false;
	}

	gfile.CloseFile();
	
	return true;
}

/*////////////////////////////////////////////////////////////////////////////////
	Zajisti vykresleni (+ flip screen)
*/
void GRun::Draw()
{
	m_view.UpdateCamera(this);
	m_view.Draw(this);
}

/*////////////////////////////////////////////////////////////////////////////////
	Zajisti pohnuti vsemi objkety
*/
void GRun::Move()
{
	// pocitac
	if (!m_multi)
		for (int i = 1; i < m_soldiers; i++)
			m_sold[i].Move(this);
	// nas hrac
	m_sold[0].Move(this);

	m_map.Move();

	m_map.ShootHits(this);
	m_map.PickPowerUps(this);
}

/*////////////////////////////////////////////////////////////////////////////////
	Testuje narazy
*/
void GRun::Crash()
{
	int k = 0, // promena cyklu
		 l = 0; // pocet narazeni
	
}

/*////////////////////////////////////////////////////////////////////////////////
	Pripravi hru na hru hrac vs. pocitace
*/
void GRun::InitSingle(int players)
{
	m_soldiers = players;
}

/*////////////////////////////////////////////////////////////////////////////////
	Zinicializuje hru
*/
bool GRun::Init(int id, int mapa)
{
	int i,j;

	bool used[AI_NAMES]; // aby se jmena neopakovala

	if (!m_map.LoadMap(mapa)) {
		allegro_message("Nepodarilo se nacist mapu. ");
		return false;
	}

	for (i = 0; i < AI_NAMES; i++)
		used[i] = false;

	// Vojak
	for(i = 0; i < MAX_PLAYERS; i++) {
		if (i == 0) m_sold[0].m_ID = id;
		else m_sold[i].m_ID = rand()%MAX_SOLDIERS;
		m_sold[i].Init(i, m_map.m_sx, m_map.m_sx, rand()%MAX_ZBRANE, true, 50, &m_map);
		m_sold[i].m_active = true;

		if (i > 0) {
			j = rand()%AI_NAMES;
			while (used[j]) j = rand()%AI_NAMES;
			strcpy(m_sold[i].m_name, ai_names[j]);
			used[j] = true;
		}
	}


	m_map.InitDone();


	m_view.Init(&m_sold[0]);

	m_map.NewPowerUp();
	m_map.NewPowerUp();
	m_map.NewPowerUp();


  if (!m_net.EnterGame(this)) {
		allegro_message("Failed to HOST or JOIN game "); // SMULA !!!!
		return false;
  }

  return true;
}

/*////////////////////////////////////////////////////////////////////////////////
	Hraje za pocitacove hrace
*/
void GRun::CompMove()
{
	int i;
	if (m_multi) return;
		for(i = 1; i < m_soldiers; i++)
			m_sold[i].CompMove(this, i);
}


/*////////////////////////////////////////////////////////////////////////////////
	Zpracovava vstupy z klavesnice
*/
void GRun::ProcessKeys()
{
	if (!keypressed()) return;

	int c;
	c = readkey() >> 8;
	switch (c) {
	case KEY_Q :
		if ((m_autorun = !m_autorun)==true)
			m_view.AddMessage("AUTORUN ZAPNUT");
		else 
			m_view.AddMessage("AUTORUN VYPNUT");
		break;
	case KEY_F3 :
		m_map.ClearGround();
		m_view.AddMessage("PROBEHLO CISTENI PODKLADU OD MRTVOL");
		break;
	case KEY_F4 :
		cdplay.NewSong();
		break;
	case KEY_F5 :
		m_map.m_mode = 0;
		break;
	case KEY_F6 :
		m_map.m_mode = 1;
		break;
	case  KEY_W :
		m_view.m_zamerovac = !m_view.m_zamerovac;
		break;
	}
}

/*////////////////////////////////////////////////////////////////////////////////
	Zinicializuje demo
*/
void GRun::InitDemo()
{
	int i,j;

	m_multi = false;

	bool used[AI_NAMES]; // aby se jmena neopakovala

	for (i = 0; i < AI_NAMES; i++)
		used[i] = false;

	m_map.LoadDemo();

	// Vojak
	for(i = 0; i < MAX_PLAYERS; i++) {
		m_sold[i].m_ID = rand()%MAX_SOLDIERS;
		m_sold[i].Init(i, m_map.m_sx, m_map.m_sx, rand()%MAX_ZBRANE, true, 50, &m_map);
		m_sold[i].m_active = true;

		if (i > 0) {
			j = rand()%AI_NAMES;
			while (used[j]) j = rand()%AI_NAMES;
			strcpy(m_sold[i].m_name, ai_names[j]);
			used[j] = true;
		}
	}

	m_soldiers = rand()%(4)+7;

	m_map.NewPowerUp();
	m_map.NewPowerUp();
	m_map.NewPowerUp();
}

/*////////////////////////////////////////////////////////////////////////////////
	Udela jeden krok dema
*/
void GRun::MoveDemo()
{
	int i;
		// zobrazeni
		 // cast pod vojaky
			m_map.Draw(320, 240, false);

			for (i = 0 ; i < m_soldiers; i++)
				m_sold[i].Draw(hscreen.m_back, 320, 240);

		// nastaveni pocitace
		CompMove();
		m_sold[0].CompMove(this, 0);

		Move();
		Crash();
}

/*////////////////////////////////////////////////////////////////////////////////
	Nacte pole bitmap (z LoadGFX)
*/
bool GRun::LoadBitmaps(BITMAP **pole, int kolik, int zacatek, char *file)
{
	PALETTE pal;
	char s[60];
	int i;

	for( i = 0; i < kolik; i++){
		// objekt
		sprintf(s, file, i+zacatek);
		*pole = load_bitmap(s,pal);
		if (!*pole) {
			allegro_message("Chybi %s",s);
			return false;
		}
		pole++;
	}
	return true;
}

/*////////////////////////////////////////////////////////////////////////////////
	Rusi pole bitmap (z Destroy)
*/
void GRun::DestroyBitmaps(BITMAP **pole, int kolik)
{
	int i;
	for( i = 0; i < kolik; i++, pole++){
		if (*pole == NULL) continue;
		destroy_bitmap(*pole);
		*pole = NULL;
	}
}

/*////////////////////////////////////////////////////////////////////////////////
	Rusi pole bitmap (z Destroy)
*/
void GRun::SetCurPlCtrl()
{

	switch(ini.ovladani) {

		// ovladani klavensnici
	case CTRL_KEYB :
		m_sold[0].SetKeys(key[KEY_UP]||key[KEY_8_PAD], 
								key[KEY_DOWN]||key[KEY_2_PAD], 
								key[KEY_LEFT]||key[KEY_4_PAD],
								key[KEY_RIGHT]||key[KEY_6_PAD], 
						      key[KEY_A]||key[KEY_J], 
								key[KEY_S]||key[KEY_K], 
								key[KEY_RSHIFT]||key[KEY_LSHIFT]||m_autorun, 
								key[KEY_RCONTROL]||key[KEY_LCONTROL], 
								key[KEY_L]||key[KEY_X]||key[KEY_ALT],
								key[KEY_ENTER]||key[KEY_SPACE]);

		m_sold[0].m_turnspeed = TURNSENSITIVE;
		break;

		// ovladani mysi
	case CTRL_KEYMOUS :
		int dx = mouse_x - 320;

		m_sold[0].SetKeys(key[KEY_UP]||key[KEY_8_PAD], 
								key[KEY_DOWN]||key[KEY_2_PAD], 
								false,
								false,
								key[KEY_LEFT]||key[KEY_4_PAD]||key[KEY_A]||key[KEY_J],
								key[KEY_RIGHT]||key[KEY_6_PAD]||key[KEY_S]||key[KEY_K], 
								key[KEY_RSHIFT]||key[KEY_LSHIFT]||m_autorun, 
								key[KEY_RCONTROL]||key[KEY_LCONTROL]||(mouse_b & 1), 
								key[KEY_L]||key[KEY_X]||key[KEY_ALT]||(mouse_b & 2),
								key[KEY_ENTER]||key[KEY_SPACE]||(mouse_b & 2));
		if (dx > 0) m_sold[0].m_right = true;
		if (dx < 0) m_sold[0].m_left  = true;
		m_sold[0].m_turnspeed = abs(dx);
		position_mouse(320, 240);
		break;
	}
}
