// GStrela.cpp: implementation of the GStrela class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "GStrela.h"
#include "GMap.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

GStrela::GStrela()
{
	m_dist = 5;
	m_active = false;
}

GStrela::~GStrela()
{

}

/*////////////////////////////////////////////////////////////////////////////////
	Inicializace parametru strely
*/
void GStrela::Init(int x, int y, int angle, int ID, int who, double speed, int zbranID)
{
	m_active = true;
	m_angleRad = angle*DPI/256;
	m_angle = angle;

	if (zbranID != -1) {
		double alpha = -atan2(Zparams[zbranID].dy, Zparams[zbranID].dx);
		double dst   = pow(pow(Zparams[zbranID].dx,2)+pow(Zparams[zbranID].dy,2),0.5);
		m_x = x + dst*cos(-m_angleRad-alpha);
		m_y = y + dst*sin(-m_angleRad-alpha);
	} 
	else {
		m_x = x;
		m_y = y;
	}

	m_ID = ID;
	m_speed = Nparams[m_ID].speed;
	if (speed > 0) m_speed += speed;
	if (Nparams[m_ID].rspeed != 0)  m_speed += rand()%Nparams[m_ID].rspeed;

	m_distance = Nparams[m_ID].dist;
	if (Nparams[m_ID].rdist != 0)  m_distance += rand()%Nparams[m_ID].rdist;

	m_dx = cos(m_angleRad) * m_speed;
	m_dy = sin(m_angleRad) * m_speed;


	m_who = who; // aby hrac netrefil sam sebe ;)
	m_network = false;
}

/*////////////////////////////////////////////////////////////////////////////////
	Pohyb strely - a pripakny vybuch :)
*/
void GStrela::Move(GMap *map)
{
	if (m_network) return;

	bool podm = map->CanShoot(m_x, m_y, m_angleRad, m_speed);

	if (podm) {
		m_x += m_dx;
		m_y -= m_dy;
		m_distance -= m_speed;
	}

	if ((m_x < -10) || (m_y < -10) ||
		(m_x > map->m_sx+10) || (m_y > map->m_sy+10)) {
		m_active = false;
		return;
	}
	if ((m_distance < 0)||(!podm)) {
			for (int i = 0; i < Nparams[m_ID].explosive; i++)
				map->NewStrela(m_x+rand()%30-15, m_y+rand()%30-15, rand()%256, Nparams[m_ID].eID, m_who, 0, -1);
			m_active = false;
			return;
	}
}

/*////////////////////////////////////////////////////////////////////////////////
	Kresleni strely
*/
void GStrela::Draw(int CX, int CY, bool light)
{
	CX = m_x - CX + (DF_X) / 2;
	CY = m_y - CY + (DF_Y)  / 2;

	if (CX > DF_X + 40 || CX < -40 ||
		 CY > DF_Y + 40 || CY < -40) return;

	rotate_sprite(hscreen.m_back, b_strel[m_ID], CX-b_strel[m_ID]->w/2, CY-b_strel[m_ID]->h/2, itofix(-m_angle));
	for (int i = 1; i < Nparams[m_ID].kour+1; i++)
		rotate_sprite(hscreen.m_back, b_kour[0], CX-b_kour[0]->w/2-cos(m_angleRad)*i*11, CY-b_kour[0]->h/2+sin(m_angleRad)*i*11, itofix(rand()%255));

	// kdyz svetlo
	if (light && Nparams[m_ID].light != -1)
		hdraw_light_sprite(hscreen.m_back, b_lights_mask[Nparams[m_ID].light], CX-b_lights_mask[Nparams[m_ID].light]->w/2, CY-b_lights_mask[Nparams[m_ID].light]->h/2);

}

/*////////////////////////////////////////////////////////////////////////////////
	Vrati poskozeni a exploduje
*/
int GStrela::GetDemage(GMap  *map)
{
	for (int i = 0; i < Nparams[m_ID].explosive; i++)
		map->NewStrela(m_x+rand()%30-15, m_y+rand()%30-15, rand()%256, Nparams[m_ID].eID, m_who, 0, -1);
	return Nparams[m_ID].demage;
}

bool GStrela::Hit(int x, int y, int dst)
{
	int sx, sy;
   int dist = m_dist + dst;

	if (!IsOut(x, y, m_x, m_y, dist)) return true; // strela zasah

	sx = m_x + cos(m_angleRad) * m_speed / 2;
	sy = m_y - cos(m_angleRad) * m_speed / 2;
	if (!IsOut(x, y, sx, sy, dist)) return true; // strela zasah

	sx = m_x - cos(m_angleRad) * m_speed / 2;
	sy = m_y + cos(m_angleRad) * m_speed / 2;
	if (!IsOut(x, y, sx, sy, dist)) return true; // strela zasah

	return false;
}

void GStrela::InitMulti(int x, int y, int angle, int ID, int who)
{
	m_active = true;
	m_angleRad = angle*DPI/256;
	m_angle = angle;

	m_x = x;
	m_y = y;

	m_ID = ID;
	m_speed = Nparams[ID].speed;
	m_distance = 50;
	m_dx = 0;
	m_dy = 0;


	m_who = who; // aby hrac netrefil sam sebe ;)
	m_network = true;
}
