/*  Hippo Games - 2000 (C) - http://www.prog.cz/hippo/
 *    __________________________________________________
 *   /\     __  __    _____   ______   ______   ______  \
 *   \ \   /\ \|\ \  /\_  _\ /\  __ \ /\  __ \ /\  __ \  \
 *    \ \  \ \ \_\ \ \//\ \/ \ \ \_\ \\ \ \_\ \\ \ \ \ \  \
 *     \ \  \ \  __ \  \ \ \  \ \  __/ \ \  __/ \ \ \ \ \  \
 *      \ \  \ \ \/\ \  \_\ \_ \ \ \/   \ \ \/   \ \ \_\ \  \
 *       \ \  \ \_\ \_\ /\____\ \ \_\    \ \_\    \ \_____\  \
 *        \ \  \/_/\/_/ \/____/  \/_/     \/_/     \/_____/   \
 *         \ \_________________________________________________\
 *          \/_________________________________________________/
 *                           
 *  
 *  Sub     : Implementation of HBitmap and HPalette classes
 *
 *  File    : HBitmap.cpp
 *  
 *  Projekt : HAllegro
 *  
 *  Autor   : Ondrej Polacek [polacek.98@gyarab.cz]
 *
 *  Datum   : X.Y.2000
 *
 */      

#include <string.h>
#include "HBitmap.h"

/* HPalette::operator= : 
 * Prizareni (kopirovani) palety
 */
HPalette& HPalette::operator= (const HPalette& pl)
{
 if (this!= &pl) memcpy(pal,pl.pal,768);
 return *this;
}

/* HPalette::set_palette : 
 * Nastavi paletu pomoci set_palette()
 */
void HPalette::set_palette()
{
 ::set_palette(pal);
}

/* HPalette::set_palette : 
 * Nacte paletu s get_palette
 */
void HPalette::get_palette()
{
 ::get_palette(pal);
}

/* HPalette::HBitmap(const char* str,HPalette &pl) : 
 * Nacte bitmapu v konstruktoru
 */
HBitmap::HBitmap(const char* str,HPalette &pl)
{
 bmp= ::load_bitmap((char*) str,(RGB*) pl);
}

/* HPalette::HBitmap(const HBitmap& bm) : 
 * Vytvori kopii bitmapy
 */
HBitmap::HBitmap(const HBitmap& bm)
{
 bmp= ::create_bitmap(bm.bmp->w,bm.bmp->h);
 ::blit(bm.bmp,bmp,0,0,0,0,bm.bmp->w,bm.bmp->h);
}

/* HPalette::HBitmap::operator= : 
 * prizareni kopirovanim
 */
HBitmap& HBitmap::operator= (const HBitmap& bm)
{
 if (this!= &bm)
  {
   if (bmp) destroy_bitmap(bmp);
   bmp= ::create_bitmap(bm.bmp->w,bm.bmp->h);
   ::blit(bm.bmp,bmp,0,0,0,0,bm.bmp->w,bm.bmp->h);
  }

 return *this;
}

/* HPalette::HBitmap::operator= : 
 * opratir porovnani
 */
int HBitmap::operator== (const HBitmap& bm)
{ return is_same_bitmap(bmp,bm.bmp); }

/* HPalette::HBitmap::operator= : 
 * opratir porovnani
 */
int HBitmap::operator== (BITMAP* bm)
{ return is_same_bitmap(bmp,bm); }

void HBitmap::Set(BITMAP* bm)
{
 if (bmp) destroy_bitmap(bmp);
 bmp= bm;
}
void HBitmap::Set(HBitmap bm)
{
 if (bm.bmp) destroy_bitmap(bmp);
 bmp= bm.bmp;
}
void HBitmap::Unset()
{ bmp= NULL; }

void HBitmap::create_bitmap(int w,int h)
{
 if (bmp) ::destroy_bitmap(bmp);
 bmp= ::create_bitmap(w,h);
}
int HBitmap::load_bitmap(const char* str, HPalette &pl)
{
 if (bmp) destroy_bitmap(bmp);
 bmp= ::load_bitmap((char*) str,(RGB*) pl);
 if (bmp) return TRUE;
     else return FALSE;
}

void HBitmap::blit(BITMAP* dest,int source_x, int source_y, int dest_x, int dest_y, int width, int height)
{
 ::blit(bmp,dest,source_x,source_y,dest_x,dest_y,width,height);
}
void HBitmap::blit(HBitmap& dest,int source_x, int source_y, int dest_x, int dest_y, int width, int height)
{
 blit(dest.bmp,source_x,source_y,dest_x,dest_y,width,height);
}
void HBitmap::blit(BITMAP* dest,int source_x, int source_y, int dest_x, int dest_y)
{
 blit(dest,source_x,source_y,dest_x,dest_y,bmp->w,bmp->h);
}
void HBitmap::blit(HBitmap& dest,int source_x, int source_y, int dest_x, int dest_y)
{
 blit(dest.bmp,source_x,source_y,dest_x,dest_y,bmp->w,bmp->h);
}
void HBitmap::masked_blit(BITMAP* dest,int source_x, int source_y, int dest_x, int dest_y, int width, int height)
{
 ::masked_blit(bmp,dest,source_x,source_y,dest_x,dest_y,width,height);
}
void HBitmap::masked_blit(HBitmap& dest,int source_x, int source_y, int dest_x, int dest_y, int width, int height)
{
 masked_blit(dest.bmp,source_x,source_y,dest_x,dest_y,width,height);
}
void HBitmap::masked_blit(BITMAP* dest,int source_x, int source_y, int dest_x, int dest_y)
{
 masked_blit(dest,source_x,source_y,dest_x,dest_y,bmp->w,bmp->h);
}
void HBitmap::masked_blit(HBitmap& dest,int source_x, int source_y, int dest_x, int dest_y)
{
 masked_blit(dest.bmp,source_x,source_y,dest_x,dest_y,bmp->w,bmp->h);
}
void HBitmap::putpixel(int _x,int _y,int col)
{
 ::putpixel(bmp,_x,_y,col);
}
void HBitmap::_putpixel(int _x,int _y,unsigned char col)
{
 ::_putpixel(bmp,_x,_y,col);
}
int HBitmap::getpixel(int _x, int _y)
{
 return ::getpixel(bmp,_x,_y);
}
unsigned char HBitmap::_getpixel(int _x, int _y)
{
 return ::_getpixel(bmp,_x,_y);
}
void HBitmap::vline(int x, int y1, int y2, int color)
{ ::vline(bmp,x,y1,y2,color); }
void HBitmap::hline(int x1, int y, int x2, int color)
{ ::hline(bmp,x1,y,x2,color); }
void HBitmap::line(int x1, int y1, int x2, int y2, int color)
{ ::line(bmp,x1,y1,x2,y2,color); }
void HBitmap::triangle(int x1,int y1,int x2,int y2,int x3,int y3, int color)
{ ::triangle(bmp,x1,y1,x2,y2,x3,y3,color); }
void HBitmap::polygon(int vertices, int *points, int color)
{ ::polygon(bmp,vertices,points,color); }
void HBitmap::rect(int x1, int y1, int x2, int y2, int color)
{ ::rect(bmp,x1,y1,x2,y2,color); }
void HBitmap::rectfill(int x1, int y1, int x2, int y2, int color)
{ ::rectfill(bmp,x1,y1,x2,y2,color); }
void HBitmap::circle(int x, int y, int radius, int color)
{ ::circle(bmp,x,y,radius,color); }
void HBitmap::circlefill(int x, int y, int radius, int color)
{ ::circlefill(bmp,x,y,radius,color); }
void HBitmap::ellipse(int x, int y, int rx, int ry, int color)
{ ::ellipse(bmp,x,y,rx,ry,color); }
void HBitmap::ellipsefill(int x, int y, int rx, int ry, int color)
{ ::ellipsefill(bmp,x,y,rx,ry,color); }
void HBitmap::arc(int x,int y, fixed ang1,fixed ang2, int r, int color)
{ ::arc(bmp,x,y,ang1,ang2,r,color); }
void HBitmap::spline(int points[8], int color)
{ ::spline(bmp,points,color); }
void HBitmap::floodfill(int x, int y, int color)
{ ::floodfill(bmp,x,y,color); }
void HBitmap::clear()
{ ::clear(bmp); }
void HBitmap::clear_to_color(int color)
{ ::clear_to_color(bmp,color); }
