//////////////////////////////////////////////////////////////////////////
//
// THE
//     ##   ## ##   ## ##  ## #####  #### ##   ##
//    #### ## ##   ## ##  ## ##     ##  ## ####
//   ## #### ##   ## #####  #####  ##  ##  ##
//  ##   ## ##   ## ##  ## ##     ##  ## ## ##
// ##   ##  #####  ##  ## #####  ##### ##    ##
//												LIBRARY
//
//////////////////////////////////////////////////////////////////////////
#ifndef NET_FROM_NUKEDX_HEADER_INCLUDED
#define NET_FROM_NUKEDX_HEADER_INCLUDED
#if _MSC_VER >= 1000
#pragma once
#endif
//////////////////////////////////////////////////////////////////////////
//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //
//////////////////////////////////////////////////////////////////////////

#define NUKEDX_VERSION "1.8 beta"
//#pragma message ( "Only for windows, because of NET")


#ifndef STRICT
#define STRICT
#endif

// Include needed headers
#include <mmsystem.h> 
#include <dplay.h>
#include <dplobby.h>


// ** Macros
#define FREE(x) if(x!=NULL){free(x);x=NULL;}

// ** Errorcodes
#define NDXERR DWORD
#define NDXERR_OK				    	 0
#define NDXERR_EVENT			   	28
#define NDXERR_NETBUSY				53
#define NDXERR_SENDMSGERR			54

// ** Forwards
class NDX_Connect;

//////////////////////////////////////////////////////////////////////////
// Class: NDX_Connect
// About: 
// Handles DirectPlay, with IPX/MODEM/SERIAL/TCPIP support. 
// Use for network games.
//////////////////////////////////////////////////////////////////////////
class NDX_Connect
{
public:
	DWORD CheckSum(LPVOID p,DWORD s);
	
	bool UseCheckSum;
	bool CheckSumError;
	
	int GetPackageSize();

	struct NetPlayer
	{
		char Name[255];
		DPID dpid;
	};
	struct NetGame
	{
		char Name[255];
		GUID guid;
	};
	struct NetConnection
	{
		void *data;
		char Name[255];
		GUID guid;
	};

	// Create the DirectPlay object, and enumerate the serviceproviders.
	bool Create(LPGUID g);

	// Connect to a service-privider.
	// Example:
	// Connect(FindSPGUID(DPSPGUID_IPX));
	bool Connect(int CNum);

	// Find a serviceprovider in the list of 'Connections'.
	int FindSPGUID(GUID spguid);

	// Connect via. modem. 'Phone' is the phonenumber to dial.
	bool ConnectMODEM(char * Phone);

	// Connect via. IPX.
	bool ConnectIPX();

	// Connect via. TCP/IP
	bool ConnectTCPIP(char *IPAddress);

	// Connect via. a null-modem/serial cabel.
	bool ConnectSERIAL(DWORD ComPort,DWORD BaudRate);

	// Close the current connection.
	bool CloseConnection();	

	// If you are the creater of the game, this functions will 
	// hide/unhide the game for other players.
	bool HideGame();
	bool UnHideGame();

	// Turn on/off guarenteed messages.
	void SetGuarenteed(bool OnOff);

	// Set/Remove an event, that does an async scan for players every 'MiliSec' miliseconds.
	NDXERR SetPlayerScanEvent(int MiliSec);
	NDXERR RemovePlayerScanEvent();
	
	// Set/Remove an event, that does an async scan for games every 'MiliSec' miliseconds.
	NDXERR SetGameScanEvent(int MiliSec);
	NDXERR RemoveGameScanEvent();	

	// Get the amount of messages waiting to be recieved.
	int GetMessageCount();
	
	// Adjust size of RBuffer, if it's too small and recieve the data.
	bool Receive();

	// Size of the recieved message.
	DWORD MessageSize;

	// An array describing the available connection/serviceproviders.
	NetConnection *Connections;

	// Amount of connection/serviceproviders in the Connections array.
	int NumConnections;
	
	// An array describing the players in the current game.
	NetPlayer *Players;

	// Amount of players in current game.
	int NumPlayers;

	// An array describing the available games.
	NetGame *Games;
	
	// Amount of available games.
	int NumGames;

	// Set the size of the RecieveBuffer.
	bool SetRBufferSize(int Size);
	
	// Size of the RecieveBuffer.
	DWORD RBufferSize;

	// Pointer to the RecieveBuffer.
	void * RBuffer;

	// Send a package.
	NDXERR Send(DPID To,LPVOID Data,DWORD DataSize);

	// Send a single DWORD.
	NDXERR Send(DPID To,DWORD dwType);
	
	// Create a new game.
	bool CreateGame(LPSTR Name,LPSTR PlayerName,int MaxPlayers);

	// Join a game.
	bool JoinGame(int GameNumber,LPSTR PlayerName);

	// Scan for available games.
	bool ScanForGames(int TimeOut,bool Async);

	// Scan for players in current game.
	bool ScanForPlayers();

	// Exit from game.
	bool CloseGame();
	
	// DPID of the package sender.
	DPID FromDPID;

	// DPID of the package reciever.
	DPID ToDPID;

	// DPID for the local player.
	DPID LocalDPID;

	NDX_Connect();
	virtual ~NDX_Connect();

	int PlayerStorage;
	int GameStorage;

private:
	LPDIRECTPLAY4A lpDP;
	void* ConnectionData;
	int MaxPlrs;
	LPGUID guid;
	DWORD GUARANTEED;
	MMRESULT PSEHandle;
	MMRESULT GSEHandle;

};

extern NDX_Connect *net;

//////////////////////////////////////////////////////////////////////////
//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //
//////////////////////////////////////////////////////////////////////////
#endif 


